/************************************************************************/
/* Copyright (c) 2003-2004 by SAS Institute Inc., Cary, NC 27513, USA   */
/*                       All Rights Reserved.                           */   
/*                                                                      */     
/* Name:          mo_create_tables                                      */
/*                                                                      */
/* Purpose:       Create MO internal metadata tables.                   */
/*                                                                      */
/* Input:         libname and table name                                */
/*                                                                      */
/* Output:        &_rc = return code                                    */
/*                                                                      */
/* Usage:         If table=ALL then all metadata tables are created.    */
/*                If table=INPUT then examples of MO input tables       */
/*                are created.                                          */
/*                                                                      */
/* Return Codes:                                                        */
/*                                                                      */
/*                rc = 0   (if successful)                              */
/*                rc = 8   (otherwise)                                  */
/*                                                                      */
/*                                                                      */
/* HISTORY:                                                             */
/* 30Jan04 S0237007 Creation (dwh)                                      */
/* 23Jul04 S0260978 Add infeasibility indicator to constraints (dwh)    */
/* 04Aug04 S0261122 Change numeric_field column length to $28. (dwh)    */
/* 11Aug04 S0256259 Add values_exist flags to project_input (dwh)       */
/*                                                                      */
/* MO 4.1 Hotfix:                                                       */
/* 17Feb05 S0284290 Use authenticated username (dwh)                    */
/*                                                                      */
/************************************************************************/

%macro mo_create_tables(lib=
                       ,table=
                       ,_rc=);

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;                                                                                                  
      %local rc;                                                                                                                      
      %let _rc = rc;                                                                                                                    
   %end;                                                                                                                                   
   %else %global &_rc;                                                                                                                     
   %let &_rc = 0;     

   %let lib=%upcase(&lib);
   %let table=%upcase(&table);
   %put;


   %mo_no_notes(off)

   %*----------------------------
    *          USERS         
    *----------------------------;
   %if (&table eq USERS or &table eq ALL) %then                          
      %do;     
         proc sql;                                       
            create table &lib..users
               (user_id               num
               ,authenticated_username char(32)
               ,username              char(32)  
               ,password              char(32)  
               ,name                  char(64)  
               ,job_title             char(64)
               ,description           char(256)  
               ,email                 char(64)  
               ,phone                 char(64)  
               ,manage_project_input  char(1)  
               ,manage_user_profiles  char(1) 
               ,manage_projects       char(1)  
               ,profile_created_by    char(64)  
              ); 
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..USERS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..USERS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *      USERS_NOTIFY         
    *----------------------------;
   %if (&table eq USERS_NOTIFY or &table eq ALL) %then 
      %do;
         proc sql;                                         
            create table &lib..users_notify
               (user_id     num
               ,notify      char(64) 
               ,active      num 
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..USERS_NOTIFY table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..USERS_NOTIFY table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *      USERS_SHARE         
    *----------------------------;
   %if (&table eq USERS_SHARE or &table eq ALL) %then 
      %do;
         proc sql;                                       
            create table &lib..users_share
               (user_id     num
               ,share       num  
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..USERS_SHARE table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..USERS_SHARE table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *      USERS_NOSHARE         
    *----------------------------;
   %if (&table eq USERS_NOSHARE or &table eq ALL) %then 
      %do;
         proc sql;                                         
            create table &lib..users_noshare
               (user_id     num
               ,noshare     num  
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..USERS_NOSHARE table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..USERS_NOSHARE table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *          UID         
    *----------------------------;
   %if (&table eq UID or &table eq ALL) %then 
      %do;
         proc sql;                                        
            create table &lib..uid
               (uid                 num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..UID table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..UID table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *          PROJECTS         
    *----------------------------;
   %if (&table eq PROJECTS or &table eq ALL) %then 
      %do;      
         proc sql;                                       
         create table &lib..projects
            (project_id                    num          
            ,user_id                       num    
            ,project_name                  char(32)    
            ,project_description           char(256) 
            ,datetime_created              num             format=datetime.  
            ,project_input_id              num          
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..PROJECTS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..PROJECTS table (sqlrc=&sqlrc).;  
      %end;      


   %*----------------------------
    *     PROJECT_INPUT        
    *----------------------------;
   %if (&table eq PROJECT_INPUT or &table eq ALL) %then 
      %do;      
         proc sql;                                       
         create table &lib..project_input
            (project_input_id                  num
            ,project_input_name                char(32)   
            ,project_input_description         char(256)   
            ,datetime_created                  num             format=datetime.
            ,data_location                     char(2048)  
            ,customer_table                    char(32)    
            ,customer_control_table            char(32)    
            ,customer_id_column                char(32)    
            ,contact_policy_segment_column     char(32)    
            ,campaign_table                    char(32)    
            ,communication_table               char(32)    
            ,channel_table                     char(32)    
            ,time_period_table                 char(32) 
            ,offer_group_values_exist          num
            ,offer_subgroup_values_exist       num
            ,time_period_values_exist          num  
            ,number_of_customers               num        
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..PROJECT_INPUT table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..PROJECT_INPUT table (sqlrc=&sqlrc).;  
      %end;



   %*----------------------------
    *          SCENARIOS         
    *----------------------------;
   %if (&table eq SCENARIOS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..scenarios
            (scenario_id                         num
            ,project_id                          num          
            ,user_id                             num    
            ,scenario_name                       char(32)
            ,scenario_description                char(256)  
            ,datetime_created                    num             format=datetime.  
            ,datetime_modified                   num             format=datetime.   
            ,status                              char(16)
            ,objective_value                     num
            ,objective_operator                  char(8)
            ,objective_attribute_name            char(32)
            ,solver_option                       num
            ,analysis_level                      num
            ,total_number_clusters               num
            ,sample                              num
            ,sample_ratio                        num
            ,sample_seed                         num	
            ,roi_numerator_attribute_name        char(32)
            ,roi_denominator_attribute_name      char(32)
            ,active_roi                          num
            ,active_season_contact_policy        num
            ,active_rolling_contact_policy       num
            ,active_offergroup_contact_policy    num
            ,active_market_seg_contact_policy    num
            ,active_budget_constraints           num
            ,active_cell_size_constraints        num
            ,active_customized_constraints       num
            ,active_chan_capacity_constraints    num
            ,active_roi_constraints              num
            ,active_suppression_rules            num
            ,constraint_summary_report_status    num
            ,summary_report_status               num
            ,histogram_report_status             num
            ,sensitivity_report_status           num
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..SCENARIOS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..SCENARIOS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *     OPTIMIZATION_JOBS         
    *----------------------------;
   %if (&table eq OPTIMIZATION_JOBS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..optimization_jobs
            (user_id                       num
            ,project_id                    num          
            ,scenario_id                   num    
            ,datetime_started              num             format=datetime.  
            ,datetime_finished             num             format=datetime.   
            ,error_code                    num
            ,pid                           num
            ,datetime_reports_finished     num             format=datetime.
            ,reports_finished              num
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OPTIMIZATION_JOBS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OPTIMIZATION_JOBS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *     MARKET_SEGMENTS         
    *----------------------------;
   %if (&table eq MARKET_SEGMENTS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..market_segments
            (project_input_id              num
            ,market_segment_id             num                
            ,market_segment                char(256)
            ,market_segment_type           num
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..MARKET_SEGMENTS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..MARKET_SEGMENTS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *     CAMPAIGNS         
    *----------------------------;
   %if (&table eq CAMPAIGNS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..campaigns
            (project_input_id              num
            ,campaign_id                   num          
            ,campaign_code                 char(32)    
            ,campaign_description          char(256)            
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CAMPAIGNS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CAMPAIGNS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *  CAMPAIGN_CONSTRAINT_RHS         
    *----------------------------;
   %if (&table eq CAMPAIGN_CONSTRAINT_RHS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..campaign_constraint_rhs
            (project_input_id              num
            ,campaign_id                   num  
            ,campaign_constraint_rhs_id    num 
            ,column_name                   char(32) 
            ,column_value                  num   
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CAMPAIGN_CONSTRAINT_RHS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CAMPAIGN_CONSTRAINT_RHS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *     COMMUNICATIONS         
    *----------------------------;
   %if (&table eq COMMUNICATIONS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..communications
            (project_input_id              num
            ,campaign_id                   num   
            ,communication_id              num       
            ,communication_code            char(32)    
            ,communication_description     char(256)            
            ,product_id                    num   
            ,outbound_channel_id           num   
            ,inbound_channel_id            num   
            ,time_period_id                num   
            ,offer_group_id                num   
            ,offer_subgroup_id             num   
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..COMMUNICATIONS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..COMMUNICATIONS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *  COMM_NUMERIC_ATTRIBUTES         
    *----------------------------;
   %if (&table eq COMM_NUMERIC_ATTRIBUTES or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..comm_numeric_attributes
            (project_input_id              num
            ,optional_comm_column_id       num       
            ,column_name                   char(32)    
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..COMM_NUMERIC_ATTRIBUTES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..COMM_NUMERIC_ATTRIBUTES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * COMM_NUMERIC_ATTRIBUTE_VALUES         
    *----------------------------;
   %if (&table eq COMM_NUMERIC_ATTRIBUTE_VALUES or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..comm_numeric_attribute_values
            (project_input_id              num
            ,campaign_id                   num
            ,communication_id              num
            ,optional_comm_column_id       num       
            ,optional_comm_column_value_id num       
            ,column_value                  num    
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..COMM_NUMERIC_ATTRIBUTE_VALUES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..COMM_NUMERIC_ATTRIBUTE_VALUES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *  COMMUNICATION_CONSTRAINT_RHS         
    *----------------------------;
   %if (&table eq COMMUNICATION_CONSTRAINT_RHS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..communication_constraint_rhs
            (project_input_id                   num
            ,communication_id                   num
            ,communication_constraint_rhs_id    num 
            ,column_name                        char(32) 
            ,column_value                       num   
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..COMMUNICATION_CONSTRAINT_RHS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..COMMUNICATION_CONSTRAINT_RHS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *    CUSTOMER_ATTRIBUTES         
    *----------------------------;
   %if (&table eq CUSTOMER_ATTRIBUTES or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..customer_attributes
            (project_input_id              num
            ,customer_attribute_id         num   
            ,customer_attribute            char(32)       
            ,attribute_type                num       
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CUSTOMER_ATTRIBUTES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CUSTOMER_ATTRIBUTES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *  CUSTOMER_OFFER_ATTRIBUTES         
    *----------------------------;
   %if (&table eq CUSTOMER_OFFER_ATTRIBUTES or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..customer_offer_attributes
            (project_input_id              num
            ,customer_offer_attribute_id   num   
            ,customer_offer_attribute      char(32)       
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CUSTOMER_OFFER_ATTRIBUTES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CUSTOMER_OFFER_ATTRIBUTES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *  CUSTOMER_ATTRIBUTE_VALUES         
    *----------------------------;
   %if (&table eq CUSTOMER_ATTRIBUTE_VALUES or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..customer_attribute_values
            (project_input_id              num
            ,customer_attribute_id         num   
            ,customer_attribute_value_id   num       
            ,customer_attribute_value      char(256)       
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CUSTOMER_ATTRIBUTE_VALUES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CUSTOMER_ATTRIBUTE_VALUES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *       CHANNELS         
    *----------------------------;
   %if (&table eq CHANNELS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..channels
            (project_input_id              num
            ,channel_id                    num   
            ,channel                       char(32)       
            ,channel_type                  num       
            ,channel_units                 char(32)       
            ,team                          char(32)       
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CUSTOMER_ATTRIBUTES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CUSTOMER_ATTRIBUTES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *  CHANNEL_CONSTRAINT_RHS         
    *----------------------------;
   %if (&table eq CHANNEL_CONSTRAINT_RHS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..channel_constraint_rhs
            (project_input_id              num
            ,channel_id                    num   
            ,channel_constraint_rhs_id     num 
            ,column_name                   char(32) 
            ,column_value                  num   
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CHANNEL_CONSTRAINT_RHS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CHANNEL_CONSTRAINT_RHS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *       PRODUCTS         
    *----------------------------;
   %if (&table eq PRODUCTS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..products
            (project_input_id              num
            ,product_id                    num   
            ,product                       char(32)       
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..PRODUCTS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..PRODUCTS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *      TIME_PERIODS         
    *----------------------------;
   %if (&table eq TIME_PERIODS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..time_periods
            (project_input_id              num
            ,time_period_id                num   
            ,time_period                   char(32) 
            ,order                         num      
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..TIME_PERIODS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..TIME_PERIODS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *      OFFER_GROUPS         
    *----------------------------;
   %if (&table eq OFFER_GROUPS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..offer_groups
            (project_input_id              num
            ,offer_group_id                num   
            ,offer_group                   char(32) 
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_GROUPS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_GROUPS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *      OFFER_SUBGROUPS         
    *----------------------------;
   %if (&table eq OFFER_SUBGROUPS or &table eq ALL) %then 
      %do;      
         proc sql;                                      
         create table &lib..offer_subgroups
            (project_input_id              num
            ,offer_group_id                num   
            ,offer_subgroup_id             num   
            ,offer_subgroup                char(32) 
            );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_SUBGROUPS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_SUBGROUPS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *       CONSTRAINTS         
    *----------------------------;
   %if (&table eq CONSTRAINTS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..constraints
               (user_id             num
               ,project_id          num
               ,scenario_id         num
               ,project_input_id    num
               ,constraint_id       num
               ,constraint_name     char(32)
               ,constraint_type     num
               ,active              num
               ,operator            char(8)      
               ,rhs                 num
               ,numeric_field       char(32)
               ,numeric_field2      char(32)
               ,offer_filter_type   num
               ,computation_type    num
               ,active_customer_filter  num
               ,active_offer_filter     num
               ,infeasible              num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CONSTRAINTS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CONSTRAINTS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * SEASONALITY_CONTACT_POLICIES         
    *----------------------------;
   %if (&table eq SEASONALITY_CONTACT_POLICIES or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..seasonality_contact_policies
               (user_id                num
               ,project_id             num
               ,scenario_id            num
               ,project_input_id       num
               ,seasonality_policy_id  num
               ,policy_name            char(32)
               ,start_period           num
               ,end_period             num
               ,max_offers             num
               ,active                 num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..SEASONALITY_CONTACT_POLICIES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..SEASONALITY_CONTACT_POLICIES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * ROLLING_CONTACT_POLICIES         
    *----------------------------;
   %if (&table eq ROLLING_CONTACT_POLICIES or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..rolling_contact_policies
               (user_id                num
               ,project_id             num
               ,scenario_id            num
               ,project_input_id       num
               ,rolling_policy_id      num
               ,policy_name            char(32)
               ,start_period           num
               ,end_period             num
               ,rolling_period_length  num
               ,max_offers             num
               ,active                 num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..ROLLING_CONTACT_POLICIES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..ROLLING_CONTACT_POLICIES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * OFFER_GROUP_CONTACT_POLICIES         
    *----------------------------;
   %if (&table eq OFFER_GROUP_CONTACT_POLICIES or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..offer_group_contact_policies
               (user_id                num
               ,project_id             num
               ,scenario_id            num
               ,project_input_id       num
               ,offer_group_policy_id  num
               ,offer_group_id         num
               ,policy_name            char(32)
               ,max_offers             num
               ,active                 num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_GROUP_CONTACT_POLICIES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_GROUP_CONTACT_POLICIES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * OFFER_SUBGROUP_CONTACT_POLICIES         
    *----------------------------;
   %if (&table eq OFFER_SUBGROUP_CONTACT_POLICIES or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..offer_subgroup_contact_policies
               (user_id                   num
               ,project_id                num
               ,scenario_id               num
               ,project_input_id          num
               ,offer_group_policy_id     num
               ,offer_subgroup_policy_id  num
               ,offer_subgroup_id         num
               ,max_offers                num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_SUBGROUP_CONTACT_POLICIES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_SUBGROUP_CONTACT_POLICIES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * MARKET_SEG_CONTACT_POLICIES         
    *----------------------------;
   %if (&table eq MARKET_SEG_CONTACT_POLICIES or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..market_seg_contact_policies
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,market_segment_policy_id      num
               ,market_segment_id             num
               ,seasonality_policy_name       char(32)
               ,rolling_policy_name           char(32)
               ,offer_group_policy_name       char(32)
               ,max_offers_policy_type        num
               ,default_max_offers_policy_id  num
               ,max_offers                    num
               ,active                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..MARKET_SEG_CONTACT_POLICIES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..MARKET_SEG_CONTACT_POLICIES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * DEFAULT_MAX_OFFERS_CONT_POLICY         
    *----------------------------;
   %if (&table eq DEFAULT_MAX_OFFERS_CONT_POLICY or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..default_max_offers_cont_policy
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,default_max_offers_policy_id  num
               ,max_offers                    num
               ,max_offers_column_name        char(32)
               ,active                        num
               ,policy_type                   num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..DEFAULT_MAX_OFFERS_CONT_POLICY table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..DEFAULT_MAX_OFFERS_CONT_POLICY table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * SUPPRESSION_RULES         
    *----------------------------;
   %if (&table eq SUPPRESSION_RULES or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..suppression_rules
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,suppression_rule_id           num
               ,suppression_rule_type         num
               ,customer_attribute_id         num
               ,customer_operator             char(8)
               ,customer_filter_value         char(256)
               ,customer_filter_value2        char(256)
               ,communication_filter_type     num
               ,customer_offer_attribute_id   num
               ,customer_offer_operator       char(8)
               ,customer_offer_filter_value   num
               ,customer_offer_filter_value2  num
               ,active                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..SUPPRESSION_RULES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..SUPPRESSION_RULES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * SUPPRESSION_CUSTOMER_FILTERS         
    *----------------------------;
   %if (&table eq SUPPRESSION_CUSTOMER_FILTERS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..suppression_customer_filters
               (user_id                           num
               ,project_id                        num
               ,scenario_id                       num
               ,suppression_rule_id               num
               ,suppression_customer_filter_id    num
               ,customer_attribute_value_id       num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..SUPPRESSION_CUSTOMER_FILTERS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..SUPPRESSION_CUSTOMER_FILTERS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * SUPPRESSION_COMMUNICATIONS         
    *----------------------------;
   %if (&table eq SUPPRESSION_COMMUNICATIONS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..suppression_communications
               (user_id                           num
               ,project_id                        num
               ,scenario_id                       num
               ,suppression_rule_id               num
               ,suppression_communication_id      num
               ,communication_id                  num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..SUPPRESSION_COMMUNICATIONS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..SUPPRESSION_COMMUNICATIONS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * OFFER_TEMPLATE_PRE_CALC_FIELDS         
    *----------------------------;
   %if (&table eq OFFER_TEMPLATE_PRE_CALC_FIELDS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..offer_template_pre_calc_fields
               (user_id                           num
               ,project_id                        num
               ,scenario_id                       num
               ,project_input_id                  num
               ,offer_template_pre_calc_field_id  num
               ,attribute_id                      num
               ,attribute_role                    num
               ,attribute_type                    num
               ,active                            num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_TEMPLATE_PRE_CALC_FIELDS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_TEMPLATE_PRE_CALC_FIELDS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * OFFER_TEMPLATE_CALC_FIELDS         
    *----------------------------;
   %if (&table eq OFFER_TEMPLATE_CALC_FIELDS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..offer_template_calc_fields
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,offer_template_calc_field_id  num
               ,attribute_name                char(32)
               ,attribute_value               char(256)
               ,attribute_role                num
               ,active                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_TEMPLATE_CALC_FIELDS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_TEMPLATE_CALC_FIELDS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * OFFER_DETAILS_CALC_FIELDS         
    *----------------------------;
   %if (&table eq OFFER_DETAILS_CALC_FIELDS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..offer_details_calc_fields
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,communication_id              num
               ,offer_template_calc_field_id  num
               ,offer_detail_calc_field_id    num
               ,attribute_value               char(256)
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_DETAILS_CALC_FIELDS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_DETAILS_CALC_FIELDS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * CHANNEL_FILTERS         
    *----------------------------;
   %if (&table eq CHANNEL_FILTERS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..channel_filters
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,constraint_id                 num
               ,channel_filter_id             num
               ,channel_id                    num
               ,active                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CHANNEL_FILTERS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CHANNEL_FILTERS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * OFFER_FILTERS         
    *----------------------------;
   %if (&table eq OFFER_FILTERS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..offer_filters
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,constraint_id                 num
               ,offer_filter_id               num
               ,offer_filter_attribute_id     num
               ,offer_filter_type             num
               ,active                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..OFFER_FILTERS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..OFFER_FILTERS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * CUSTOMER_FILTERS         
    *----------------------------;
   %if (&table eq CUSTOMER_FILTERS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..customer_filters
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,project_input_id              num
               ,constraint_id                 num
               ,customer_filter_id            num
               ,customer_attribute_id         num
               ,customer_attribute_value_id   num
               ,active                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CUSTOMER_FILTERS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CUSTOMER_FILTERS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * REPORTVARS         
    *----------------------------;
   %if (&table eq REPORTVARS or &table eq ALL) %then 
      %do;      
         proc sql;                                    
            create table &lib..reportvars
               (user_id                       num
               ,project_id                    num
               ,scenario_id                   num
               ,reportvar_id                  num
               ,var                           char(32)
               ,solution                      num
               ,report                        num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..REPORTVARS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..REPORTVARS table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    * REPORTS         
    *----------------------------;

   %if (&table eq REPORTS or &table eq ALL) %then 
      %do;      
          /* report_code: 0 - objective summary  */
	      /*              1 - constraint summary */
          /*              2 - offer summary      */
	      /*              3 - histogram          */
	      /*              4 - sensitivity        */
	      /* report_type: 0 - table, 1 - graph   */

          proc sql noprint;                                                                                                             
            create table &lib..reports                                                                                                       
               (report_name                   char(32)                                                                                  
               ,report_id                     num                                                                                       
               ,report_type                   num                                                                                       
               ,report_code                   num                                                                                       
               );                                                                                                                       
                                                                                                                                        
            insert into &lib..reports                                                                                                        
               set                                                                                                                      
                  report_name   = 'Objective summary'                                                                                  
                 ,report_id     = 1                                                                                                     
                 ,report_type   = 0                                                                                                     
                 ,report_code   = 0                                                                                                     
               set                                                                                                                      
                  report_name   = 'Constraint summary'                                                                                  
                 ,report_id     = 2                                                                                                     
                 ,report_type   = 0                                                                                                     
                 ,report_code   = 1                                                                                                     
               set                                                                                                                      
                  report_name   = 'Offer summary'                                                                                       
                 ,report_id     = 3                                                                                                     
                 ,report_type   = 0                                                                                                     
                 ,report_code   = 2                                                                                                     
               set                                                                                                                      
                  report_name   = 'Offer summary'                                                                                       
                 ,report_id     = 4                                                                                                     
                 ,report_type   = 1                                                                                                     
                 ,report_code   = 2                                                                                                     
               set                                                                                                                      
                  report_name   = 'Histogram'                                                                                           
                 ,report_id     = 5                                                                                                     
                 ,report_type   = 1                                                                                                     
                 ,report_code   = 3                                                                                                     
               set                                                                                                                      
                  report_name   = 'Sensitivity'                                                                                         
                 ,report_id     = 6                                                                                                     
                 ,report_type   = 1                                                                                                     
                 ,report_code   = 4                                                                                                     
               ;                                                                                                                        
          quit;                          
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..REPORTS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..REPORTS table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq REPORTS_COMPARE or &table eq ALL) %then 
      %do;      
          /* report_code: 0 - objective summary  */
	      /*              1 - constraint summary */
          /*              2 - offer summary      */
	      /* report_type: 0 - table, 1 - graph   */

          proc sql noprint;                                                                                                             
            create table &lib..reports_compare                                                                                                       
               (report_name                   char(32)                                                                                  
               ,report_id                     num                                                                                       
               ,report_type                   num                                                                                       
               ,report_code                   num                                                                                       
               );                                                                                                                       
                                                                                                                                        
            insert into &lib..reports_compare                                                                                                        
               set                                                                                                                      
                  report_name   = 'Objective summary'                                                                                  
                 ,report_id     = 1                                                                                                     
                 ,report_type   = 0                                                                                                     
                 ,report_code   = 0                                                                                                     
               set                                                                                                                      
                  report_name   = 'Constraint summary'                                                                                  
                 ,report_id     = 2                                                                                                     
                 ,report_type   = 0                                                                                                     
                 ,report_code   = 1                                                                                                     
               set                                                                                                                      
                  report_name   = 'Offer summary'                                                                                       
                 ,report_id     = 3                                                                                                     
                 ,report_type   = 0                                                                                                     
                 ,report_code   = 2                                                                                                     
               set                                                                                                                      
                  report_name   = 'Offer summary'                                                                                       
                 ,report_id     = 4                                                                                                     
                 ,report_type   = 1                                                                                                     
                 ,report_code   = 2                                                                                                     
               ;                                                                                                                        
          quit;                          
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..REPORTS_COMPARE table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..REPORTS_COMPARE table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq REPORT_FILTERS or &table eq ALL) %then 
      %do;      
	      /* filter_value_type: -1 - over all     */
	      /*                    0 - campaign      */
	      /*                    1 - communication */
	      /*                    2 - product       */
	      /*                    3 - outbound      */
	      /*                    4 - inbound       */
	      /*                    5 - offer group   */
	      /*                    6 - offer subgroup*/
  	      /*                    7 - time period   */

          proc sql noprint;                                                                                                                  
            create table &lib..report_filters                                                                                                
               (filter_name                   char(32)                                                                                       
               ,filter_id                     num                                                                                            
               ,report_id                     num                                                                                            
               ,filter_type                   num                                                                                            
               );                                                                                                                            
                                                                                                                                             
            insert into &lib..report_filters                                                                                                 
               set                                                                                                                           
                  filter_name   = 'Offer filter'                                                                                             
                 ,filter_id     = 1                                                                                                          
                 ,report_id     = 3                                                                                                          
                 ,filter_type   = 0                                                                                                          
               set                                                                                                                           
                  filter_name   = 'Offer filter'                                                                                             
                 ,filter_id     = 2                                                                                                          
                 ,report_id     = 4                                                                                                          
                 ,filter_type   = 0                                                                                                          
               set                                                                                                                           
                  filter_name   = 'Measure filter'                                                                                           
                 ,filter_id     = 3                                                                                                          
                 ,report_id     = 4                                                                                                          
                 ,filter_type   = 1                                                                                                          
               set                                                                                                                           
                  filter_name   = 'Offer filter'                                                                                             
                 ,filter_id     = 4                                                                                                          
                 ,report_id     = 5                                                                                                          
                 ,filter_type   = 0                                                                                                          
               ;                                                                                                                             
          quit;       
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..REPORT_FILTERS table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..REPORT_FILTERS table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq REPORT_FILTER_VALUES or &table eq ALL) %then 
      %do;      
          /* filter_value_type: -1 - over all     */
	      /*                    0 - campaign      */
	      /*                    1 - communication */
	      /*                    2 - product       */
	      /*                    3 - outbound      */
	      /*                    4 - inbound       */
	      /*                    5 - offer group   */
	      /*                    6 - offer subgroup*/
	      /*                    7 - time period   */

          proc sql noprint;                                                                                                             
            create table &lib..report_filter_values                                                                                                
               (filter_value_name             char(32)                                                                                  
               ,filter_value_id               num                                                                                       
               ,filter_id                     num                                                                                       
               ,filter_value_type             num                                                                                       
               );                                                                                                                       
                                                                                                                                        
            insert into &lib..report_filter_values                                                                                                 
               set                                                                                                                      
                  filter_value_name   = 'Campaign'                                                                                      
                 ,filter_value_id     = 1                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 1                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Communication'                                                                                 
                 ,filter_value_id     = 2                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 2                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Product'                                                                                       
                 ,filter_value_id     = 3                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 3                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Channel'                                                                         
                 ,filter_value_id     = 4                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 4                                                                                               
               set
                  filter_value_name   = 'Offer group'                                                                                   
                 ,filter_value_id     = 5                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 5                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Offer subgroup'                                                                                
                 ,filter_value_id     = 6                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 6                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Time period'                                                                                   
                 ,filter_value_id     = 7                                                                                               
                 ,filter_id           = 1                                                                                               
                 ,filter_value_type   = 7                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Campaign'                                                                                      
                 ,filter_value_id     = 8                                                                                               
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 1                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Communication'                                                                                 
                 ,filter_value_id     = 9                                                                                              
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 2                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Product'                                                                                       
                 ,filter_value_id     = 10                                                                                              
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 3                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Channel'                                                                         
                 ,filter_value_id     = 11                                                                                              
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 4                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Offer group'                                                                                   
                 ,filter_value_id     = 12                                                                                              
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 5                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Offer subgroup'                                                                                
                 ,filter_value_id     = 13                                                                                              
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 6                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Time period'                                                                                   
                 ,filter_value_id     = 14                                                                                              
                 ,filter_id           = 2                                                                                               
                 ,filter_value_type   = 7                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Offers'                                                                                        
                 ,filter_value_id     = 15                                                                                              
                 ,filter_id           = 3                                                                                               
                 ,filter_value_type   = 0                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Objective'                                                                                     
                 ,filter_value_id     = 16                                                                                              
                 ,filter_id           = 3                                                                                               
                 ,filter_value_type   = 1                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'ROI'                                                                                           
                 ,filter_value_id     = 17                                                                                              
                 ,filter_id           = 3                                                                                               
                 ,filter_value_type   = 2                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Overall'                                                                                       
                 ,filter_value_id     = 18                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 0                                                                                              
               set                                                                                                                      
                  filter_value_name   = 'Campaign'                                                                                      
                 ,filter_value_id     = 19                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 1                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Communication'                                                                                 
                 ,filter_value_id     = 20                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 2                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Product'                                                                                       
                 ,filter_value_id     = 21                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 3                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Channel'                                                                         
                 ,filter_value_id     = 22                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 4                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Offer group'                                                                                   
                 ,filter_value_id     = 23                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 5                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Offer subgroup'                                                                                
                 ,filter_value_id     = 24                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 6                                                                                               
               set                                                                                                                      
                  filter_value_name   = 'Time period'                                                                                   
                 ,filter_value_id     = 25                                                                                              
                 ,filter_id           = 4                                                                                               
                 ,filter_value_type   = 7                                                                                               
               ;                                                                                                                        
          quit;                                                                                                                         
          %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..REPORT_FILTER_VALUES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..REPORT_FILTER_VALUES table (sqlrc=&sqlrc).;  
      %end;


   %*----------------------------
    *         PROPERTIES         
    *----------------------------;
   %if (&table eq PROPERTIES or &table eq ALL) %then                          
      %do;     
         proc sql;                                       
            create table &lib..properties
               (key                    char(32)
               ,value                  char(256)  
               ); 

            insert into &lib..properties
               set key="STORED_PROCESS_PATH",                                                                                                            
                   value="file:&mostp";       

            insert into &lib..properties
               set key="SERVER_PLATFORM",
               %if (%upcase(&operating_system) eq WIN) %then %do;
                   value="WINDOWS";  
               %end;    
               %else %do;  
                   value="UNIX";
               %end;      
    
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..PROPERTIES table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..PROPERTIES table (sqlrc=&sqlrc).;  
      %end;



  %*---------------------------------------------------------------------------------------
   *        
   *---------------------------------------------------------------------------------------;







   %*----------------------------
    * INPUT DATA TABLES         
    *----------------------------;
   %if (&table eq CAMPAIGN_INPUT or &table eq INPUT) %then 
      %do;      
         proc sql;                                    
            create table &lib..campaign
               (camp_code                     char(32)
               ,camp_desc                     char(256)
               ,camp_max_budget               num
               ,camp_min_budget               num
               ,camp_max_cell_size            num
               ,camp_min_cell_size            num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CAMPAIGN input table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CAMPAIGN input table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq COMMUNICATION_INPUT or &table eq INPUT) %then 
      %do;
         proc sql;                                    
            create table &lib..communication
               (camp_code                     char(32)
               ,comm_code                     char(32)
               ,comm_desc                     char(256)
               ,comm_unit_cost                num
               ,product                       char(32)
               ,outbound_channel              char(32)
               ,outbound_team                 char(32)
               ,outbound_unit_usage           num
               ,inbound_channel               char(32)
               ,inbound_team                  char(32)
               ,inbound_unit_usage            num               
               ,comm_max_budget               num
               ,comm_min_budget               num
               ,comm_max_cell_size            num
               ,comm_min_cell_size            num
               ,offer_group                   char(32)
               ,offer_subgroup                char(32)
               ,time_period                   char(32)
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..COMMUNICATION input table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..COMMUNICATION input table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq CHANNEL_INPUT or &table eq INPUT) %then 
      %do;      
         proc sql;                                    
            create table &lib..channel
               (channel                       char(32)
               ,team                          char(32)
               ,units                         char(32)
               ,type                          num
               ,min_usage                     num
               ,max_usage                     num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CHANNEL input table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CHANNEL input table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq TIME_PERIOD_INPUT or &table eq INPUT) %then 
      %do;      
         proc sql;                                    
            create table &lib..time_period
               (time_period                   char(32)
               ,order                         num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..TIME_PERIOD input table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..TIME_PERIOD input table (sqlrc=&sqlrc).;  
      %end;


   %if (&table eq CUSTOMER_CONTROL_INPUT or &table eq INPUT) %then 
      %do;      
         proc sql;                                    
            create table &lib..customer_control
               (column_name                   char(32)
               ,numeric_field                 char(28)
               ,camp_code                     char(32)
               ,comm_code                     char(32)
               ,type                          num
               );
         quit;
         %if (&sqlrc eq 0) %then %do;  
            %if (%upcase(&lib) ne WORK) %then %put %mo_note Created the &lib..CUSTOMER_CONTROL input table.;  
         %end;
         %else %put %mo_note Error encountered creating the &lib..CUSTOMER_CONTROL input table (sqlrc=&sqlrc).;  
      %end;

   %mo_no_notes(on)   



   %*--------------------------------;                                                                                                                            
   %* Exit processing                ;                                                                                                                            
   %*--------------------------------;                                                                      
   %* Successful completion; 
   %put;                                                                                                                           
   %goto EXIT;          

   %ERROR_EXIT:
   %let &_rc = 8;
   %put %mo_error Error encountered during creation of table (rc=&&&_rc).;                                                                       
                                                                                                                                                                          
   %EXIT:      

%mend mo_create_tables;
